#!/bin/sh 

# BIG BROTHER MONITORING SCRIPT
#
# For each system we monitor we watch
#	disk	USES df 
#	cpu	USES uptime
#	procs	CHECKS A LIST
# 	msgs	LOOKS FOR WARNINGS
#
# For the network we care about
#	telnet	USES telnet and checks the results
#	http	USES lynx to check for valid output
#
# THEN SEND THE RESULTS TO THE DISPLAY SERVER USING bb

# echo "***** BBHOME IS SET TO $BBHOME"

. $BBHOME/etc/bbdef.sh		# INCLUDE STANDARD DEFINITIONS

#
#=====================================================================
# DISK TEST
#=====================================================================
#
set `$DF | $GREP $DFUSE | $GREP -v $DFEXCLUDE | $SORT +${DFSORT} | $TAIL -1`
DISKNAME=$6
DISKPCT=`echo $5 | $SED "s/%//"`

# echo "$DISKNAME $DISKPCT"

if test "$DISKPCT" -ge "$DFPANIC"
then
	COLOR="red"
	if test "$DFPAGE" = "Y"		# CALL THE PAGER
	then
		$BB $BBPAGE "page $PAGER ${DISKERR}${MACHIP}"
	fi
elif test "$DISKPCT" -ge "$DFWARN"
then
	COLOR="yellow"
else
	COLOR="green"
fi

#
# NOW SEND THIS INFORMATION TO THE BIG BROTHER DISPLAY UNIT
#
$BB $BBDISP "status $MACHINE.disk $COLOR `date` $DISKNAME is ${DISKPCT}% full"

#
#=====================================================================
# CPU TEST
#=====================================================================
#
set `$UPTIME`
shift;shift;shift;shift;
ARG=`expr $# - 2`			# 5 MINUTE LOAD AVERAGE
COUNT=1
while test "$COUNT" -le "$ARG"
do
	shift;
	COUNT=`expr $COUNT + 1`
done

AVG=`echo $1 | $SED "s/\,//" | $SED "s/\.//" `
AVG=`expr $AVG + 0`
# echo "LOAD AVERAGE: $AVG"

if test "$AVG" -ge "$CPUPANIC"
then
	COLOR="red"
	if test "$CPUPAGE" = "Y"         # CALL THE PAGER
	then
		$BB $BBPAGE "page $PAGER ${CPUERR}${MACHIP}"
	fi
elif test "$AVG" -ge "$CPUWARN"
then
	COLOR="yellow"
else	
	COLOR="green"
fi

$PS > $BBTMP/bb.$$			# GET A PS LISTING
P=`$CAT $BBTMP/bb.$$ | $WC`		# NUMBER OF PROCESSES
P=`echo $P`
U=`$WHO | $WC`				# GET NUMBER OF USERS
U=`echo $U`

#
# NOW SEND THIS INFORMATION TO THE BIG BROTHER DISPLAY UNIT
#
$BB $BBDISP "status $MACHINE.cpu $COLOR `date` $U users, $P procs, load=${AVG}"

#
#=====================================================================
# PROCESSES THAT MUST EXIST
#=====================================================================
#

COLOR="green"

for proc in $PROCS
do
	$GREP "$proc" $BBTMP/bb.$$ | $GREP -v grep > /dev/null 2>&1
	if test "$?" != "0"
	then
		LINE="$LINE - $proc DOWN"
		#
		# NOW SEE IF THIS IS A PROCESS WE SHOULD PAGE ON...
		# BUT ONLY IF A PAGER IS ACTUALLY DEFINED...
		#
		echo "$PAGEPROC" | $GREP "$proc" > /dev/null 2>&1
		if test "$?" = "0"
		then
			if test "$PAGER" != ""
			then
				LINE="${LINE} (paging)"
				$BB $BBPAGE "page $PAGER ${PROCERR}${MACHIP}"
			fi
			COLOR="red"
		fi	
	else
		LINE="$LINE - $proc OK"
	fi
done

#
# IF SOMETHING IS DOWN, AND IT'S NOT CRITICAL, JUST WARN
#
echo "$LINE" | $GREP "DOWN" > /dev/null 2>&1
if test "$?" = "0"				# SOMETHING IS DOWN
then
	if test "$COLOR" = "green"		# WASN'T A PANIC
	then
		COLOR="yellow"			# SO JUST WARN
	fi
fi
#
# NOW SEND THIS INFORMATION TO THE BIG BROTHER DISPLAY UNIT
#
$BB $BBDISP "status $MACHINE.procs $COLOR `date` $LINE"
$RM -f $BBTMP/bb.$$

#
#=====================================================================
# NOTIFY ON BAD MESSAGES
#=====================================================================
#
COLOR="green"

for word in $MSGS
do
	LINE=`$GREP "$word" $MSGFILE`
	if test "$LINE" != ""			# FOUND A BAD WORD
	then
		echo "$PAGEMSG" | $GREP "$word" > /dev/null 2>&1
		if test "$?" = "0"
		then
			COLOR="red"
			if test "$PAGER" != ""
			then
				LINE="${LINE} (paging)"
				$BB $BBPAGE "page $PAGER ${PROCERR}${MACHIP}"
			fi
			$BB $BBDISP "status $MACHINE.msgs $COLOR `date` $LINE"
			break;			# ENOUGH ALREADY...
		else
			if test "$COLOR" = "green"
			then
				COLOR="yellow"
				$BB $BBDISP "status $MACHINE.msgs $COLOR `date` $LINE"
			fi
		fi
	fi
done

if test "$COLOR" = "green"
then
	$BB $BBDISP "status $MACHINE.msgs $COLOR `date` No bad messages"
fi

